﻿using System;
using System.Windows.Forms;
using CSCore;
using CSCore.CoreAudioAPI;
using CSCore.SoundIn;
using CSCore.SoundOut;
using CSCore.Streams;
using FragLabs.Audio.Codecs;

namespace OpusDemo
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            using (var deviceEnumerator = new MMDeviceEnumerator())
            using (
                var deviceCollection = deviceEnumerator.EnumAudioEndpoints(DataFlow.Capture, DeviceState.Active)
                   )
            {
                foreach (var device in deviceCollection)
                {
                    comboBox1.Items.Add(device.FriendlyName);
                }
            

            if (comboBox1.Items.Count > 0)
                comboBox1.SelectedIndex = 0;

                using (
                    var deviceCollection2 = deviceEnumerator.EnumAudioEndpoints(DataFlow.Render, DeviceState.Active)
                    )
                {
                    foreach (var device in deviceCollection2)
                    {
                        comboBox2.Items.Add(device.FriendlyName);
                    }
                }
                if (comboBox2.Items.Count > 0)
                comboBox2.SelectedIndex = 0;
            }
        }

        private void button1_Click(object sender, EventArgs e)
        {
            button2.Enabled = true;
            button1.Enabled = false;
            StartEncoding();
        }

        private void button2_Click(object sender, EventArgs e)
        {
            button1.Enabled = true;
            button2.Enabled = false;
            StopEncoding();
        }

        WasapiCapture _waveIn;
        WasapiOut _waveOut;
        OpusEncoder _encoder;
        OpusDecoder _decoder;
        int _segmentFrames;
        int _bytesPerSegment;
        ulong _bytesSent;
        DateTime _startTime;
        Timer _timer;
        private WriteableBufferingSource _writeableBufferingSource;

        void StartEncoding()
        {
            _startTime = DateTime.Now;
            _bytesSent = 0;
            _segmentFrames = 960;
            _encoder = OpusEncoder.Create(48000, 1, FragLabs.Audio.Codecs.Opus.Application.Voip);
            _encoder.Bitrate = 8192;
            _decoder = OpusDecoder.Create(48000, 1);
            _bytesPerSegment = _encoder.FrameByteCount(_segmentFrames);

            MMDevice captureDevice = null;
            using (var deviceEnumerator = new MMDeviceEnumerator())
            using (
                var deviceCollection = deviceEnumerator.EnumAudioEndpoints(DataFlow.Capture, DeviceState.Active)
                )
            {
                foreach (var device in deviceCollection)
                {
                    if (device.FriendlyName == comboBox1.Text)
                    {
                        captureDevice = device;
                        break;
                    }
                }
            }

            if (captureDevice == null)
                return;

            //var waveFormatExtensible = new WaveFormatExtensible(44100, 32, 2, Guid.Parse("00000003-0000-0010-8000-00aa00389b71"));

            var waveFormatExtensible = new WaveFormat(48000, 16, 1);

            _waveIn = new WasapiCapture(true, AudioClientShareMode.Shared, 100, waveFormatExtensible)
            {
                Device = captureDevice
            };
            _waveIn.Initialize();
            _waveIn.DataAvailable += _waveIn_DataAvailable1;

            _writeableBufferingSource = new WriteableBufferingSource(waveFormatExtensible) {FillWithZeros = true};

            _waveOut = new WasapiOut();
            _waveOut.Initialize(_writeableBufferingSource);

            _waveOut.Play();
            _waveIn.Start();

            if (_timer == null)
            {
                _timer = new Timer();
                _timer.Interval = 1000;
                _timer.Tick += _timer_Tick;
            }
            _timer.Start();
        }

        private void _waveIn_DataAvailable1(object sender, DataAvailableEventArgs e)
        {
            _writeableBufferingSource.Write(e.Data, e.Offset, e.ByteCount);
            return;
            byte[] soundBuffer = new byte[e.ByteCount + _notEncodedBuffer.Length];
            for (int i = 0; i < _notEncodedBuffer.Length; i++)
                soundBuffer[i] = _notEncodedBuffer[i];
            for (int i = 0; i < e.ByteCount; i++)
                soundBuffer[i + _notEncodedBuffer.Length] = e.Data[i];

            int byteCap = _bytesPerSegment;
            int segmentCount = (int)Math.Floor((decimal)soundBuffer.Length / byteCap);
            int segmentsEnd = segmentCount * byteCap;
            int notEncodedCount = soundBuffer.Length - segmentsEnd;
            _notEncodedBuffer = new byte[notEncodedCount];
            for (int i = 0; i < notEncodedCount; i++)
            {
                _notEncodedBuffer[i] = soundBuffer[segmentsEnd + i];
            }

            for (int i = 0; i < segmentCount; i++)
            {
                byte[] segment = new byte[byteCap];
                for (int j = 0; j < segment.Length; j++)
                    segment[j] = soundBuffer[i * byteCap + j];
                int len;
                byte[] buff = _encoder.Encode(segment, segment.Length, out len);
                _bytesSent += (ulong)len;
                buff = _decoder.Decode(buff, len, out len);
                _writeableBufferingSource.Write(buff, 0, len);
            }
        }

        void _timer_Tick(object sender, EventArgs e)
        {
            var timeDiff = DateTime.Now - _startTime;
            var bytesPerSecond = _bytesSent / timeDiff.TotalSeconds;
            Console.WriteLine("{0} Bps", bytesPerSecond);
        }

        byte[] _notEncodedBuffer = new byte[0];

        void StopEncoding()
        {
            _timer.Stop();
            _waveIn.Stop();
            _waveIn.Dispose();
            _waveIn = null;
            _waveOut.Stop();
            _waveOut.Dispose();
            _waveOut = null;
            _writeableBufferingSource.Dispose();
            _writeableBufferingSource = null;
            _encoder.Dispose();
            _encoder = null;
            _decoder.Dispose();
            _decoder = null;
        }
    }
}
